from odps.udf import annotate

@annotate('string,binary->string')
class ExtractPDFPages(object):
    
    def __init__(self):
        import sys
        sys.path.insert(0, "work/pdf_tools_cp37.tar.gz/packages")

    def evaluate(self, key, file_content):
        
        import base64
        import os
        import oss2
        import fitz
        import json
        from tqdm import tqdm
        
        try:
            endpoint = "oss-cn-hangzhou.aliyuncs.com"
            auth = oss2.Auth("LTAI5tEhtzhcCBJjWRw18wb9", "Ceq7YKY2mHxymMoTspQAJZrVnXaZjD")
            bucket = oss2.Bucket(auth, endpoint, "odps-external-qltest")
            target_folder = 'qltest/dailyrunnew/mengdou_test_object_table/test1/pdfs'
            pdf_base_name = os.path.basename(key)

            def rect_to_dict(rect):
                return {
                    'x0': rect.x0,
                    'y0': rect.y0,
                    'x1': rect.x1,
                    'y1': rect.y1,
                }
            
            ret = {
                'file_path': key,
                'metadata': None,
                'pages': [],
                'pngs': []
            }
            
            total_pages = 0
            with fitz.open(stream=file_content, filetype="pdf") as doc:
                total_pages = len(doc)
                ret['metadata'] = doc.metadata
                ret['xml_metadata'] = doc.get_xml_metadata()
                for page in tqdm(doc):
                    # extract info
                    page_data = {}
                    page_data['number'] = page.number
                    page_data['text'] = page.get_text().strip()
                    page_data['images'] = page.get_images()
                    page_data['image_rects'] = [[rect_to_dict(_) for _ in page.get_image_rects(elem)] for elem in page_data['images']]
                    del page_data['images']

                    page_data['fonts'] = page.get_fonts()
                    try:
                        page_data['num_drawings'] = len(page.get_drawings())
                    except Exception as e:
                        page_data['num_drawings'] = None
                    try:
                        page_data['num_tables'] = len(page.find_tables())
                    except Exception as e:
                        page_data['num_tables'] = None
                    ret['pages'].append(page_data)
                    
                    # generate png
                    upscale = 2
                    zoom_x = upscale
                    zoom_y = upscale
                    mat = fitz.Matrix(zoom_x, zoom_y)
                    pix = page.get_pixmap(matrix=mat)
                    png_content = pix.tobytes('png')
                    bucket.put_object(os.path.join(target_folder, 'extracted_pngs', pdf_base_name , f"page_{page.number}.png"), png_content)
                    
            ret = json.dumps(ret, ensure_ascii=False)

            # upload results
            bucket.put_object(os.path.join(target_folder, 'extracted_contents', pdf_base_name,'result.txt'), ret)

            return f"success:{total_pages} pages"
        except Exception as e:
            return f"error:{str(e)}"
